@@ -1,21 +1,7 @@
-﻿/*
-    Copyright 2019 Harald Sitter <sitter@kde.org>
+/*
+    SPDX-FileCopyrightText: 2019 Harald Sitter <sitter@kde.org>
 
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of the GNU General Public License as
-    published by the Free Software Foundation; either version 2 of
-    the License or (at your option) version 3 or any later version
-    accepted by the membership of KDE e.V. (or its successor approved
-    by the membership of KDE e.V.), which shall act as a proxy
-    defined in Section 14 of version 3 of the license.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */
 
 #include "key.h"
@@ -62,7 +48,7 @@ static QString keySymToString(KeySym keysym)
 {
     // Strangely enough xkbcommons's UTF map is incomplete with regards to
     // dead keys. Extend it a bit.
-    static QHash<unsigned long, char> deadMap{
+    static QHash<unsigned long, char16_t> deadMap{
         {XK_dead_grave, 0x0060},
         {XK_dead_acute, 0x00b4},
         {XK_dead_circumflex, 0x02c6},
@@ -78,7 +64,7 @@ static QString keySymToString(KeySym keysym)
         {XK_dead_ogonek, 0x02DB},
         {XK_dead_iota, 0x0269},
         {XK_dead_voiced_sound, 0x309B},
-        {XK_dead_semivoiced_sound, 0x309A},
+        {XK_dead_semivoiced_sound, 0x309C},
         {XK_dead_belowdot, 0x0323},
         {XK_dead_hook, 0x0309},
         {XK_dead_horn, 0x031b},
@@ -131,8 +117,17 @@ static QString keySymToString(KeySym keysym)
     if (static_cast<KeySym>(xkbKeysym) == keysym) {
         str = xkbKeysymToUtf8(xkbKeysym);
 
-        for (const auto &c : qAsConst(str)) {
-            if (!c.isPrint() && !c.isNull()) {
+        const auto ucs4Str = str.toUcs4();
+
+        if (!str.isEmpty()) {
+            const QChar::Category category = QChar::category(ucs4Str[0]);
+            if (category == QChar::Mark_NonSpacing || category == QChar::Mark_Enclosing) {
+                str.prepend(" ");
+            }
+        }
+
+        for (const auto &c : qAsConst(ucs4Str)) {
+            if (!QChar::isPrint(c) && !(c == 0) && !(QChar::category(c) == QChar::Other_PrivateUse)) {
                 str = "";
                 break;
             }
@@ -151,10 +146,27 @@ static QString keySymToString(KeySym keysym)
     }
 
     if (deadMap.contains(keysym)) {
-        str = QChar(deadMap[keysym]);
+        const QChar c(deadMap[keysym]);
+
+        // Make sure non-spacing diacritics are rendered
+        if (c.category() == QChar::Mark_NonSpacing || c.category() == QChar::Mark_Enclosing) {
+            // FIXME: should really be NBSP, but it doesn't seem to
+            // render properly, so use SPACE which is not recommended
+            // since Unicode 4.1
+            str = QStringLiteral(" ") + c;
+        } else {
+            str = c;
+        }
     }
 
-    return str.replace('_', ' ');
+    // X11 keys can be of the form "Control_L".
+    // Split them so they are easier on the eyes.
+    // But only do that on strings of 3 chars or more to not lose "_"
+    if (str.size() > 2) {
+        str.replace('_', ' ');
+    }
+
+    return str;
 }
 
 KeyCap::KeyCap(const QString symbols[], QObject *parent)
@@ -181,7 +193,7 @@ Key::Key(XkbKeyPtr key_, XkbDescPtr xkb_, QObject *parent)
         Q_ASSERT(event);
         if (event->nativeScanCode() == nativeScanCode) {
             pressed = event->type() == QKeyEvent::KeyPress;
-            emit pressedChanged();
+            Q_EMIT pressedChanged();
         }
     });
 }
